
PURPLE = '\033[95m'
CYAN = '\033[96m'
DARKCYAN = '\033[36m'
BLUE = '\033[94m'
GREEN = '\033[92m'
YELLOW = '\033[93m'
RED = '\033[91m'
BOLD = '\033[1m'
UNDERLINE = '\033[4m'
END = '\033[0m'

START_MSG = 'Starting console interface'
LOGO_TITLE = ' Main menu '


ENTER_PROMPT = 'Input'

EXIT_MSG = 'Exiting program'
INVALID_TERMINAL_INPUT_MSG = 'Invalid input'
ERROR_INDICATOR = BOLD + RED + f"{' ERROR ':#^40}" + END

INPUT_BLANK_HINT_RETURN = 'Enter blank to return to parent menu'
INPUT_BLANK_HINT_PASS = ''
INPUT_BLANK_HINT_EXIT = 'Enter blank to exit program'

INPUT_WAIT_PROMPT_MSG = 'Press enter to continue'
